/* File: FlipHack.h
   ================
   This is the public header file for FlipHack.  It contains those things that 
   someone would need if they wanted to call the exported code from outside
   FlipHack.

   History
   -------
   10/31/99 Created
 */

#ifndef _flip_hack_h
#define _flip_hack_h

// ----------------------------------
// Constants...
// ----------------------------------

/* Hack creator -- used in macros below */
#define kFlipHackCreatorCode			'RFlp'
#define kFlipHackType					'HACK'

/* Helper code ID -- used in macros below */
#define kFlipHackHelperCodeID			0


// ----------------------------------
// Error definitions...
// ----------------------------------

typedef enum {
	kFlipHackErrGeneric = appErrorClass,
	kFlipHackErrCouldntUninstall,
	kFlipHackErrOsNotApproved,
	kFlipHackErrCpuNotApproved,
	kFlipHackErrBitDepthTooHigh,
	kFlipHackErrScreenNotSquare,
	kFlipHackErrUnknownGraphics,
	kFlipHackErrCouldntChangeAngle,
	kFlipHackErrNotEnoughMemory,
	kFlipHackNotInstalled
} FlipHackErrorEnum;

// ----------------------------------
// Types...
// ----------------------------------

/* Different angles that we can work at */
typedef enum { 
	kFlipNoAngle = -1, kFlip0Deg = 0, kFlip90Deg, kFlip180Deg, kFlip270Deg, kFlipNumAngles
} FlipAngleT;


// ----------------------------------
// Access to helper functions...
// ----------------------------------

/* List of available helper functions */
typedef enum {
	kCheckForCompatibilityHelperId,
	kGetAngleHelperId, 
	kSetAngleHelperId
} HelperFunctionIdEnum;

/* This function is used to get access to other helper functions */
typedef void* (*GetHelperFunctionFn) (Word /* HelperFunctionIdEnum */ functionId);

/* Prototypes of all helper functions */
typedef Err (*CheckForCompatibilityHelperFn) (Boolean allowUserInteraction);
typedef Byte /* FlipAngleT */ (*GetAngleHelperFn) (void);
typedef Err (*SetAngleHelperFn) (FlipAngleT* angleP);



/* Macro to make it easy to call CheckForCompatibility() helper function.  This is a 
   bit slow, but is easy to use. */
#define CheckForCompatibility(/* Boolean */ allowUserInteraction, /* Err* */ errP)														\
do {																							\
	DmOpenRef openRef;																			\
																								\
	openRef = DmOpenDatabaseByTypeCreator (kFlipHackType, kFlipHackCreatorCode, dmModeReadOnly);\
	if (openRef != NULL) {																		\
		VoidHand codeHandle;																	\
																								\
		codeHandle = DmGet1Resource ('code', kFlipHackHelperCodeID);							\
		if (codeHandle != NULL) {																\
			VoidPtr codePtr = MemHandleLock (codeHandle);										\
																								\
			if (codePtr != NULL) {																\
				GetHelperFunctionFn getHelperFunctionFn;										\
				CheckForCompatibilityHelperFn checkForCompatibilityHelperFn;					\
																								\
				getHelperFunctionFn = (GetHelperFunctionFn) codePtr;							\
				checkForCompatibilityHelperFn = (CheckForCompatibilityHelperFn) 				\
										getHelperFunctionFn (kCheckForCompatibilityHelperId);	\
																								\
				*(errP) = checkForCompatibilityHelperFn((allowUserInteraction));				\
																								\
				MemHandleUnlock (codeHandle);													\
			}																					\
																								\
			DmReleaseResource (codeHandle);														\
		}																						\
		DmCloseDatabase (openRef);																\
	}																							\
} while (0)


/* Macro to make it easy to call GetAngle() helper function.  This is a bit slow, but 
   is easy to use. */
#define GetAngle(/* FlipAngleT* */ angleP)														\
do {																							\
	DmOpenRef openRef;																			\
																								\
	openRef = DmOpenDatabaseByTypeCreator (kFlipHackType, kFlipHackCreatorCode, dmModeReadOnly);\
	if (openRef != NULL) {																		\
		VoidHand codeHandle;																	\
																								\
		codeHandle = DmGet1Resource ('code', kFlipHackHelperCodeID);							\
		if (codeHandle != NULL) {																\
			VoidPtr codePtr = MemHandleLock (codeHandle);										\
																								\
			if (codePtr != NULL) {																\
				GetHelperFunctionFn getHelperFunctionFn;										\
				GetAngleHelperFn getAngleHelperFn;												\
																								\
				getHelperFunctionFn = (GetHelperFunctionFn) codePtr;							\
				getAngleHelperFn = (GetAngleHelperFn) getHelperFunctionFn (kGetAngleHelperId);	\
																								\
				*(angleP) = (FlipAngleT) getAngleHelperFn();									\
																								\
				MemHandleUnlock (codeHandle);													\
			}																					\
																								\
			DmReleaseResource (codeHandle);														\
		}																						\
		DmCloseDatabase (openRef);																\
	}																							\
} while (0)


/* Macro to make it easy to call SetAngle() helper function.  This is a bit slow, but 
   is easy to use.  Err may be NULL if you don't care about errors... */
#define SetAngle(/* FlipAngleT* */ angleP, /* Err* */ errP)										\
do {																							\
	DmOpenRef openRef;																			\
																								\
	openRef = DmOpenDatabaseByTypeCreator (kFlipHackType, kFlipHackCreatorCode, dmModeReadOnly);\
	if (openRef != NULL) {																		\
		VoidHand codeHandle;																	\
																								\
		codeHandle = DmGet1Resource ('code', kFlipHackHelperCodeID);							\
		if (codeHandle != NULL) {																\
			VoidPtr codePtr = MemHandleLock (codeHandle);										\
																								\
			if (codePtr != NULL) {																\
				GetHelperFunctionFn getHelperFunctionFn;										\
				SetAngleHelperFn setAngleHelperFn;												\
																								\
				getHelperFunctionFn = (GetHelperFunctionFn) codePtr;							\
				setAngleHelperFn = (SetAngleHelperFn) getHelperFunctionFn (kSetAngleHelperId);	\
																								\
				if ((errP) == NULL) {															\
					(void) setAngleHelperFn ((angleP));											\
				} else {																		\
					*((Err*)(errP)) = setAngleHelperFn ((angleP));								\
				}																				\
																								\
				MemHandleUnlock (codeHandle);													\
			}																					\
																								\
			DmReleaseResource (codeHandle);														\
		}																						\
		DmCloseDatabase (openRef);																\
	}																							\
} while (0)


#endif // _flip_hack_h
